'use strict';
var libQ = require('kew');
var config = new (require('v-conf'))();
var fs = require('fs-extra');
var ioctl = require('ioctl');
var ref = require('ref');
var ArrayType = require('ref-array');
var StructType = require('ref-struct');
var videodev2 = require("./videodev2");
module.exports = ControllerV4LRadio;

// define the "v4l2_capability" struct type
var v4l2_capability = StructType({
	driver : ArrayType(ref.types.char, 16),
	card : ArrayType(ref.types.char, 32),
	bus_info : ArrayType(ref.types.char, 32),
	version : ref.types.uint32,
	capabilities : ref.types.uint32,
	device_caps : ref.types.uint32,
	reserved : ArrayType(ref.types.uint32, 3)
});


// define the "v4l2_tuner" struct type
var v4l2_tuner = StructType({
	index : ref.types.uint32,
	name : ArrayType(ref.types.char, 32),
	type : ref.types.uint32,
	capability : ref.types.uint32,
	rangelow : ref.types.uint32,
	rangehigh : ref.types.uint32,
	rxsubchans : ref.types.uint32,
	audmode : ref.types.uint32,
	signal : ref.types.int32,
	afc : ref.types.uint32,
	reserved : ArrayType(ref.types.uint32, 4)
});

// define the "v4l2_frequency" struct type
var v4l2_frequency = StructType({
	tuner : ref.types.uint32,
	type : ref.types.uint32,
	frequency : ref.types.uint32,
	reserved : ArrayType(ref.types.uint32, 8)
});

function ControllerV4LRadio(context) {
	var self = this;


	this.context = context;
	this.commandRouter = this.context.coreCommand;
	this.logger = this.context.logger;
	this.configManager = this.context.configManager;
	self.radio_fd = -1
}

ControllerV4LRadio.prototype.getConfigurationFiles = function()
{
	return ['config.json'];
}

ControllerV4LRadio.prototype.addToBrowseSources = function () {
	var data = {name: 'V4L Radio', uri: 'v4l_radio',plugin_type:'music_service',plugin_name:'v4l_radio'};
	this.commandRouter.volumioAddToBrowseSources(data);
};

ControllerV4LRadio.prototype.removeToBrowseSources = function () {
	var data = {name: 'V4L Radio', uri: 'v4l_radio',plugin_type:'music_service',plugin_name:'v4l_radio'};
	this.commandRouter.volumioRemoveToBrowseSources(data);
};

// Plugin methods -----------------------------------------------------------------------------

ControllerV4LRadio.prototype.openRadioDevice = function() {
	var self = this;
	var defer = libQ.defer();
	var radio_fd;

	if(self.radio_fd != -1) {
		self.logger.info("ControllerV4LRadio Radio device already opened");
	}

	fs.open(self.config.get('radiodevice'), 'r+', function(err, radio_fd) {
		if (err) {
			self.logger.info("ControllerV4LRadio openRadioDevice Error");
			return defer.reject(new Error('Cannot open device file'));
		}
		self.radio_fd = radio_fd;
			self.logger.info("ControllerV4LRadio openRadioDevice");
		defer.resolve();
	});

	return defer.promise;
}

ControllerV4LRadio.prototype.closeRadioDevice = function() {
	var self = this;
	var defer = libQ.defer();

	if(self.radio_fd != -1) {
		fs.close(self.radio_fd, function(err) {
			if (err) {
				self.logger.info("ControllerV4LRadio closeRadioDevice Error");
				return defer.reject(new Error('Cannot close device file'));
			}
			self.radio_fd = -1;
			self.logger.info("ControllerV4LRadio closeRadioDevice");
			defer.resolve();
		});
	}

	return defer.promise;
}

ControllerV4LRadio.prototype.setFrequency = function(newFrequency) {
	var self = this;
	var frequency = new v4l2_frequency();

	frequency.tuner = 0;
	frequency.type = videodev2.V4L2_TUNER_RADIO;
	frequency.frequency = newFrequency;

	if(self.radio_fd != -1) {
		self.logger.info("ControllerV4LRadio setFrequency: " + frequency.frequency);
		ioctl(self.radio_fd, videodev2.VIDIOC_S_FREQUENCY >>> 0, frequency.ref());
		if (err) {
			self.logger.info("ControllerV4LRadio setFrequency Error");
		}
	}
}

ControllerV4LRadio.prototype.getFrequency = function() {
	var self = this;
	var frequency = new v4l2_frequency();

	frequency.tuner = 0;
	frequency.type = videodev2.V4L2_TUNER_RADIO;

	if(self.radio_fd != -1) {
		var ret = ioctl(self.radio_fd, videodev2.VIDIOC_G_FREQUENCY >>> 0, frequency.ref());
		self.logger.info("ControllerV4LRadio getFrequency ret: " + ret);
		self.logger.info("ControllerV4LRadio getFrequency tuner: 0x" + frequency.tuner.toString(16));
		self.logger.info("ControllerV4LRadio getFrequency type: 0x" + frequency.type.toString(16));
		self.logger.info("ControllerV4LRadio getFrequency frequency: 0x" + frequency.frequency.toString(16));
	}
}

ControllerV4LRadio.prototype.getTuner = function() {
	var self = this;
	var tuner = new v4l2_tuner();

	if(self.radio_fd != -1) {
		tuner.index = 0;
		tuner.type = videodev2.V4L2_TUNER_RADIO;
		var ret = ioctl(self.radio_fd, videodev2.VIDIOC_G_TUNER >>> 0, tuner.ref());
		self.logger.info("ControllerV4LRadio getTuner ret: " + ret);
		self.logger.info("ControllerV4LRadio getTuner index: 0x" + tuner.index.toString(16));
		self.logger.info("ControllerV4LRadio getTuner name: " + tuner.name.buffer.toString());
		self.logger.info("ControllerV4LRadio getTuner type: 0x" + tuner.type.toString(16));
		self.logger.info("ControllerV4LRadio getTuner capability: 0x" + tuner.capability.toString(16));
		self.logger.info("ControllerV4LRadio getTuner rangelow: 0x" + tuner.rangelow.toString(16));
		self.logger.info("ControllerV4LRadio getTuner rangehigh: 0x" + tuner.rangehigh.toString(16));
		self.logger.info("ControllerV4LRadio getTuner rxsubchans: 0x" + tuner.rxsubchans.toString(16));
		self.logger.info("ControllerV4LRadio getTuner audmode: 0x" + tuner.audmode.toString(16));
		self.logger.info("ControllerV4LRadio getTuner signal: 0x" + tuner.signal.toString(16));
		self.logger.info("ControllerV4LRadio getTuner afc: 0x" + tuner.afc.toString(16));
	}
}

ControllerV4LRadio.prototype.queryCap = function() {
	var self = this;
	var capability = new v4l2_capability();

	if(self.radio_fd != -1) {
		var ret = ioctl(self.radio_fd, videodev2.VIDIOC_QUERYCAP >>> 0, capability.ref());
		self.logger.info("ControllerV4LRadio queryCap ret: " + ret);
		self.logger.info("ControllerV4LRadio queryCap driver: " + capability.driver.buffer.toString());
		self.logger.info("ControllerV4LRadio queryCap card: " + capability.card.buffer.toString());
		self.logger.info("ControllerV4LRadio queryCap bus_info: " + capability.bus_info.buffer.toString());
		self.logger.info("ControllerV4LRadio queryCap version: 0x" + capability.version.toString(16));
		self.logger.info("ControllerV4LRadio queryCap capabilities: 0x" + capability.capabilities.toString(16));
		self.logger.info("ControllerV4LRadio queryCap device_caps: 0x" + capability.device_caps.toString(16));
	}
}

ControllerV4LRadio.prototype.onStart = function() {
	var self = this;
	var defer = libQ.defer();

	this.mpdPlugin=this.commandRouter.pluginManager.getPlugin('music_service', 'mpd');
	self.logger.info("ControllerV4LRadio onStart");
	self.openRadioDevice()
		.then(function() {
			self.logger.info("ControllerV4LRadio openRadioDevice success");
			self.addToBrowseSources();
			self.queryCap();
			self.getTuner();
			self.closeRadioDevice()
				.then(function() {
					self.logger.info("ControllerV4LRadio closeRadioDevice success");
				})
			defer.resolve();
		})
		.fail(function() {
			self.logger.info("ControllerV4LRadio openRadioDevice error");
			self.closeRadioDevice();
			defer.reject(new Error());
		});

	return defer.promise;
};

ControllerV4LRadio.prototype.onStop = function () {
	var self = this;
	var defer = libQ.defer();

	self.logger.info("ControllerV4LRadio onStop");
	this.removeToBrowseSources();
	return defer.resolve();
};

ControllerV4LRadio.prototype.rewriteForUri = function (array) {
    var self = this;
    var lastPlayed = array;

    for(var i = 0; i < lastPlayed.length; i++){
        lastPlayed[i].type = "v4l_radio";
    }

    return lastPlayed;
}

ControllerV4LRadio.prototype.handleBrowseUri=function(curUri) {
    var self = this;
    var response = [];
    var stationList = [];
    var defer = libQ.defer();


        self.mpdPlugin.sendMpdCommand('add alsa://' + self.config.get('alsadevice'),[])
        .then(function()
        {
            self.commandRouter.stateMachine.setConsumeUpdateService('mpd');
            return self.mpdPlugin.sendMpdCommand('play',[]);
        });

		self.openRadioDevice()
			.then(function() {
				self.logger.info("ControllerV4LRadio openRadioDevice success");
				defer.resolve();
			})
			.fail(function() {
				self.logger.info("ControllerV4LRadio openRadioDevice error");
				self.closeRadioDevice();
				defer.reject(new Error());
			});



    try {
        stationList = fs.readJsonSync('/data/stationlist.json', {throws: false});
        stationList = self.rewriteForUri(stationList);
        response = {
            navigation: {
                prev: {
                    uri: "/"
                },
                lists: [{
                    "availableListViews": ["list"],
                    "items": stationList
                }]
            }
        };
        defer.resolve(response)
    } catch(e) {
        console.log('Error Retrieving Station List file: '+e);
        response = {
            navigation: {
                prev: {
                    uri: "/"
                },
                lists: [{
                    "availableListViews": ["list"],
                    "items": []
                }]
            }
        };
        defer.resolve(response)
    }

    return defer.promise;
}


//self.setFrequency(5550); //88800/16
//self.commandRouter.executeOnPlugin('music_service', 'mpd', 'addPlay', 'alsa://hw:1,0');

/*
 * This method can be defined by every plugin which needs to be informed of the startup of Volumio.
 * The Core controller checks if the method is defined and executes it on startup if it exists.
 */
ControllerV4LRadio.prototype.onVolumioStart = function () {
	var self = this;
	var configFile = this.commandRouter.pluginManager.getConfigurationFile(this.context,'config.json');
	this.config = new (require('v-conf'))();
	this.config.loadFile(configFile);

	return libQ.resolve();
};

ControllerV4LRadio.prototype.onRestart = function () {
	var self = this;
	//Perform restart tasks here
};

ControllerV4LRadio.prototype.onInstall = function () {
	var self = this;
	//Perform your installation tasks here
};

ControllerV4LRadio.prototype.onUninstall = function () {
	var self = this;
	//Perform your deinstallation tasks here
};

ControllerV4LRadio.prototype.getUIConfig = function () {
	var defer = libQ.defer();
	var self = this;

	var lang_code = this.commandRouter.sharedVars.get('language_code');

	self.commandRouter.i18nJson(__dirname+'/i18n/strings_'+lang_code+'.json',
		__dirname+'/i18n/strings_en.json',
		__dirname + '/UIConfig.json')
		.then(function(uiconf) {
			uiconf.sections[0].content[0].value = self.config.get('radiodevice');
			uiconf.sections[0].content[1].value = self.config.get('alsadevice');
			defer.resolve(uiconf);
		})
		.fail(function() {
			defer.reject(new Error());
		});

	return defer.promise;
};

ControllerV4LRadio.prototype.setUIConfig = function (data) {
	var self = this;
	//Perform your UI configuration tasks here
};

ControllerV4LRadio.prototype.getConf = function (varName) {
	var self = this;
	//Perform your tasks to fetch config data here
};

ControllerV4LRadio.prototype.setConf = function (varName, varValue) {
	var self = this;
	//Perform your tasks to set config data here
};

ControllerV4LRadio.prototype.saveRadioConfiguration = function (data) {
	var self = this;
	var defer = libQ.defer();

	self.logger.info("ControllerV4LRadio saveRadioConfiguration");
	self.config.set('radiodevice', data['radiodevice']);
	self.config.set('alsadevice', data['alsadevice']);

	self.commandRouter.pushToastMessage('success', "Configuration update", 'The configuration has been successfully updated');

	defer.resolve({});
	return defer.promise;
};

//Optional functions exposed for making development easier and more clear
ControllerV4LRadio.prototype.getSystemConf = function (pluginName, varName) {
	var self = this;
	//Perform your tasks to fetch system config data here
};

ControllerV4LRadio.prototype.setSystemConf = function (pluginName, varName) {
	var self = this;
	//Perform your tasks to set system config data here
};

ControllerV4LRadio.prototype.getAdditionalConf = function () {
	var self = this;
	//Perform your tasks to fetch additional config data here
};

ControllerV4LRadio.prototype.setAdditionalConf = function () {
	var self = this;
	//Perform your tasks to set additional config data here
};
